use rug::Integer;
use crate::matrix::Matrix;
use lll_rs::l2::bigl2;
mod lll;
mod matrix;

fn abs(i: Integer) -> Integer {
    if i < 0 {
        -i
    } else {
        i
    }
}

fn main() {
    // 1. Build lattice matrix basis
    let ciphertexts = vec![
Integer::from(37459),
Integer::from(8227),
Integer::from(44119),
Integer::from(22575),
Integer::from(9249),
Integer::from(38483),
Integer::from(26181),
    ];
    let noise_bits = 2;
    let basis_matrix = Matrix::new_lattice(noise_bits, ciphertexts.clone()).unwrap();

    // println!("matrix: {:?}", basis_matrix);

    // 2. reduce with LLL
    let mut lll_matrix = basis_matrix.to_lll_matrix();
    println!("matrix: {:?}", lll_matrix);
    bigl2::lattice_reduce(&mut lll_matrix, 0.51, 0.75);
    println!("matrix: {:?}", lll_matrix);

    // 3. Extract shortest vector
    let shortest_vector = &lll_matrix[0];
    println!("Shortest vector: {:?}", shortest_vector);

    // 4. q0 candidate
    let q0 = &shortest_vector[0] / (Integer::from(1) << (noise_bits + 1));
    println!("q0: {}", q0);

    // 5. Find p
    // compute r0 = x0 (mod q0)
    // and p = (x0 − r0)/q0.
    let x0 = &ciphertexts[0];
    println!("x0: {}", x0);
    let r0 = x0 % q0.clone();
    println!("r0: {}", r0);
    let p_guess = abs((x0 - r0) / q0);

    println!("Recovered p: {}", p_guess);
}
